/*************************************************************************/
/*************************************************************************/
/* H. Timucin OZDEMIR                                                    */
/* Date : February, 1997                                                 */
/* File Name : JSSPChromosomesImpl.cc                                    */
/*************************************************************************/
/*************************************************************************/

#include <stdio.h>
#include <iostream.h>
#include "JSSPChromosomes.h"
#include <intFitnessValue.h>
#include <intFitnessBag.h>
#include <FitnessValue.h>
#include <FitnessValueBag.h>
#include <trapMF.h>
#include <gaussMF.h>
#include <limits.h>
#include "../deffunc.h"

JSSPChromosomes::JSSPChromosomes() : JSSPSolution() {}

JSSPChromosomes::~JSSPChromosomes()
{ myProblem = NULL;
  cout << " de allocate JSSPChromosomes " << endl;
  if(FacilityIndex) delete [] FacilityScheduleList;
  FacilityIndex = 0;
}// end of destructor

JSSPChromosomes::JSSPChromosomes(const JSSPChromosomes& RHS)
{ myProblem = RHS.myProblem ;
  cout << " copy constructor  JSSPChromosomes " << endl;
  FacilityIndex = RHS.FacilityIndex;
  FacilityScheduleList = new FacilitySchedule[FacilityIndex];
  for(int i=0;i<FacilityIndex;i++) 
      FacilityScheduleList[i] = RHS.FacilityScheduleList[i];
  valid = 0;
}// end of COPY constructor


FitnessValue * JSSPChromosomes::getFitnessValue()
{  intFitnessValue *Fv = new intFitnessValue(INT_MAX);
   //if(!isFeasible()) restore();
   (*Fv).setFitnessValue((float)getMakespan());
   return Fv;
}// end of getFitnessValue()

FitnessValueBag * JSSPChromosomes::getFitnessValueBag()
{  intFitnessBag *Fv = new intFitnessBag();
   cout << " in JSSPChromosomes::getFitnessValueBag() " << endl;
   return Fv;
}// end of getFitnessValueBag()

void JSSPChromosomes::randomInitialize()
{  cout << " Random initialization " << endl;  
   solveRandom();
   cout << " Random solution is created " << endl;
}// end of randomInitialize()

void JSSPChromosomes::printOn(ostream& os) const
{  os << (*(JSSPSolution *)this);
}

float JSSPChromosomes::HammingDistance(Chromosomes *curr) 
{  JSSPChromosomes *k = (JSSPChromosomes *) curr;
   int i,j, diff=0, totGene=0;
   FacilitySchedule *h1;
   FacilitySchedule *h2;
   Operation * o1;
   Operation * o2;

   for(i=0;i<(*k).getMaxFacilityNumber();i++)
   {  
      h1 = (*k).getFacilityScheduleIndex(i);
      h2 = getFacilityScheduleIndex(i);

      /*
      if(h1!=NULL&&h2==NULL) diff++;
      else if(h1==NULL&&h2!=NULL) diff++;
      else if(h1&&h2&&(!((*h1)==(*h2)))) diff++;
      */

      if(h1&&h2)
      totGene += minimum((*h1).getNumberOfOperations(),(*h2).getNumberOfOperations());
      if(h1&&h2)
        for(j=0;j<minimum((*h1).getNumberOfOperations(),(*h2).getNumberOfOperations());j++)
        {  o1 = (*h1).getOperationWithIndex(j);   // Get the oper. of First
	   o2 = (*h2).getOperationWithIndex(j);   // Get the oper. of Second
	   if(o1==NULL&&o2!=NULL) diff++;         // One of them NULL
	   else if(o1!=NULL&&o2==NULL) diff++;    // One of them NULL
	   else if(o1&&o2&&(!(*o1==*o2))) diff++; // They are not IDENTICAL
        }// end of for

   }// end of for

   float res=(float)diff/(float)totGene;
   cout << " Hamming distance is " << diff ;
   cout << " TotGene : " << totGene << " Norm Hamming is " << res << endl;
   return res;

}// end of HammingDistance()

JSSPChromosomes::FindSimilarClass(Chromosomes *curr) 
{  JSSPChromosomes *k = (JSSPChromosomes *) curr;
   int i,j, diff=0, totGene=0;
   FacilitySchedule *h1;
   FacilitySchedule *h2;
   Operation * o1;
   Operation * o2;

   for(i=0;i<(*k).getMaxFacilityNumber();i++)
   {  
      h1 = (*k).getFacilityScheduleIndex(i);
      h2 = getFacilityScheduleIndex(i);

      if(h1!=NULL&&h2==NULL) diff++;
      else if(h1==NULL&&h2!=NULL) diff++;
      else if(h1&&h2&&(!((*h1)==(*h2)))) diff++;

      /*
      if(h1&&h2)
        for(j=0;j<min((*h1).getNumberOfOperations(),(*h2).getNumberOfOperations());j++)
        {  o1 = (*h1).getOperationWithIndex(j);   // Get the oper. of First
	   o2 = (*h2).getOperationWithIndex(j);   // Get the oper. of Second
	   if(o1==NULL&&o2!=NULL) diff++;         // One of them NULL
	   else if(o1!=NULL&&o2==NULL) diff++;    // One of them NULL
	   else if(o1&&o2&&(!(*o1==*o2))) diff++; // They are not IDENTICAL
        }// end of for
      */

   }// end of for

   gaussMF none(0.0,0.05);
   gaussMF verySimilar(0.1,0.08);
   gaussMF similar(0.4,0.29);
   trapMF  muchAlike(0.65,0.85,0.99,1.0);

   totGene = (*k).getMaxFacilityNumber();
   /*
   for(i=0;i<(*k).getMaxFacilityNumber();i++)
   {  h1 = (*k).getFacilityScheduleIndex(i);
      if(h1) totGene+=(*h1).getNumberOfOperations();
   }// end of for
   */

   float Mem[4];
   float val = (float)diff/(float)totGene;
   Mem[0] = none.getValue(val);
   Mem[1] = verySimilar.getValue(val);
   Mem[2] = similar.getValue(val);
   Mem[3] = muchAlike.getValue(val);

   float maxi;
   i=0; maxi=Mem[0];
   for(j=1;j<4;j++) if(maxi<Mem[j]) { i=j; maxi=Mem[j]; } 

   cout << " val : " << val << " similarity is " << maxi << " i : " << i << endl;
   return(i);

}// end of FindSimilarClass()

JSSPChromosomes::similar(Chromosomes *curr) 
{  int simClass = FindSimilarClass(curr);  
   return(simClass==1||simClass==2?1:0);
}// end of similar()

JSSPChromosomes::disSimilar(Chromosomes *curr) 
{  int simClass = FindSimilarClass(curr);  
   return(simClass==3?1:0);
}// end of disSimilar()

JSSPChromosomes::operator==(const JSSPChromosomes& RHS)
{
    return isEqual(RHS);

}// end of operator==()


JSSPChromosomes::isEqual(const Chromosomes& RHS)
{  JSSPChromosomes *k = (JSSPChromosomes *) &RHS;  
   FacilitySchedule *s1;
   FacilitySchedule *s2;
   int ret = 1;
   int i;

   //ret = (getMyProblem()==(*k).getMyProblem());
   ret = ( ret && getMaxFacilityNumber() == (*k).getMaxFacilityNumber());

   for(i=0;ret&&i<getMaxFacilityNumber();i++)
   {  s1 = getFacilityScheduleIndex(i);
      s2 = (*k).getFacilityScheduleIndex(i);
      ret = (ret && (*s1) == (*s2) );
      if(!ret) cout << i << "th Fac. Sched is different " << endl;
   }// end of for

   cout << " I am out of JSSPChromosomes::isEqual " << endl;

   return ret;

}// end of isEqual()
